//#################################################################


Lib.Gui = (function() {

    var obj = {};

    var repo = Lib.createRepo();

    //Simple Text Display

    function Text(opts) {
        var self = this;
        this.x = Lib.check.isNumber(opts.x) ? opts.x : 0;
        this.y = Lib.check.isNumber(opts.y) ? opts.y : 0;
        this.text = Lib.check.totalStringCheck(opts.text, "STANDARD GUI TEXT");
        this.style = Lib.check.isObject(opts.style) ? opts.style : {
            font: "20px Arial",
            fill: "#ffffff"
        };
        var group = Lib.check.objHasMethod(opts.group, "create") ? opts.group : undefined;
        var anchor = Lib.check.objHasProps(opts.anchor, ["x", "y"]) ? opts.anchor : new Phaser.Point(1, 0);


        // this.obj = game.add.text(this.x, this.y, this.text, this.style);
        this.obj = Lib.UTILS.createTextInGroup(this.x, this.y, this.text, this.style, group);
        this.obj.anchor.setTo(anchor.x, anchor.y);

        this.setText = function(text) {
            self.obj.text = text;
        }

        // STANDARD REPO CODE - ----------------------------------------
        repo.addObj(this);
        //REQUIRED-----------------
        this.destroy = function() {
                repo.removeObj(this);
                self.obj.destroy();
            }
            //OPTIONAL--------------------
        this._update = function() {
            // p("Im a button");
        }


    }
    obj.Text = Text;

    //one time text that appears, travells upwards and self destroys
    function TravellingText(opts) {
        var self = this;
        this.x = Lib.check.isNumber(opts.x) ? opts.x : 0;
        this.y = Lib.check.isNumber(opts.y) ? opts.y : 0;
        this.text = Lib.check.totalStringCheck(opts.text, "STANDARD GUI TEXT");
        this.style = Lib.check.isObject(opts.style) ? opts.style : {
            font: "20px Arial",
            fill: "#ffffff"
        };

        travelDistance = Lib.check.isNumber(opts.travelDistance) ? opts.travelDistance : -100;
        var anchor = Lib.check.objHasProps(opts.anchor, ["x", "y"]) ? opts.anchor : new Phaser.Point(0.5, 0.5);
        var group = Lib.check.objHasMethod(opts.group, "create") ? opts.group : undefined;


        this.obj = Lib.UTILS.createTextInGroup(this.x, this.y, this.text, this.style, group);
        this.obj.anchor.setTo(anchor.x,anchor.y);

        //TWEEN
        var travellingTween = game.add.tween(this.obj);
        // var travellingEvent = undefined;

        travellingTween.to({
            alpha: 1
        }, Phaser.Timer.SECOND * 0.2, Phaser.Easing.Quadratic.Out);

        travellingTween.to({
            y: this.y + travelDistance,
            alpha: 0
        }, Phaser.Timer.SECOND * 2, Phaser.Easing.Quadratic.Out);

        travellingTween.onComplete.add(_destroyObject, self);

        travellingTween.start();

        function _destroyObject(){
            self.destroy();
        }


        // STANDARD REPO CODE - ----------------------------------------
        repo.addObj(this);
        //REQUIRED-----------------
        this.destroy = function() {
                repo.removeObj(this);
                self.obj.destroy();

                if (Lib.check.exists(travellingTween)) {
                    travellingTween.stop();
                }

            }
            //OPTIONAL--------------------
        this._update = function() {
            // p("Im a button");
        }

    }
    obj.TravellingText = TravellingText;

    function ScoreText(opts) {
        var self = this;
        this.x = Lib.check.isNumber(opts.x) ? opts.x : 0;
        this.y = Lib.check.isNumber(opts.y) ? opts.y : 0;
        this.style = Lib.check.isObject(opts.style) ? opts.style : {
            font: "20px Arial",
            fill: "#ffffff"
        };
        this.valueGetter = Lib.check.isFunction(opts.valueGetter) ? opts.valueGetter : function() {
            return -1
        };
        var group = Lib.check.objHasMethod(opts.group, "create") ? opts.group : undefined;

        this.obj = Lib.UTILS.createTextInGroup(this.x, this.y, "NO TEXT", this.style, group);
        this.obj.anchor.setTo(1, 0);

        var currentValue = self.valueGetter();
        var diff = 0;
        var travelSpeed = 10;

        this.travelToTarget = function(target) {
            if (currentValue < target) {
                diff = target - currentValue;
            }

            if (diff < 5)
                currentValue += Lib.UTILS.DELTA_TIME() * travelSpeed * 2;
            else if (diff > 5 && diff < 10)
                currentValue += Lib.UTILS.DELTA_TIME() * travelSpeed * 4;
            else if (diff > 10 && diff < 20)
                currentValue += Lib.UTILS.DELTA_TIME() * travelSpeed * 8;
            else if (diff > 20 && diff < 50)
                currentValue += Lib.UTILS.DELTA_TIME() * travelSpeed * 16;
            else if (diff > 50 && diff < 150)
                currentValue += Lib.UTILS.DELTA_TIME() * travelSpeed * 32;
            else if (diff > 150 && diff < 300)
                currentValue += Lib.UTILS.DELTA_TIME() * travelSpeed * 40;
            else if (diff > 300 && diff < 1000)
                currentValue += Lib.UTILS.DELTA_TIME() * travelSpeed * 400;
            else if (diff > 1000)
                currentValue += Lib.UTILS.DELTA_TIME() * travelSpeed * 500;
            else
                currentValue += Lib.UTILS.DELTA_TIME() * travelSpeed;

            if (currentValue > target) {
                currentValue = target;
            }

        }



        // STANDARD REPO CODE - ----------------------------------------
        repo.addObj(this);
        //REQUIRED-----------------
        this.destroy = function() {
                repo.removeObj(this);
                self.obj.destroy();
            }
            //OPTIONAL--------------------
        this._update = function() {
            // p("Im a button");
            self.obj.text = Math.round(currentValue).toString();
            self.travelToTarget(self.valueGetter());
        }

    }
    obj.ScoreText = ScoreText;


    //GUI IMAGE---------------------------------------------
    //no input on this one
    function Image(opts) {
        var self = this;
        var opts = opts || {}; //default
        this.x = Lib.check.isNumber(opts.x) ? opts.x : 0;
        this.y = Lib.check.isNumber(opts.y) ? opts.y : 0;
        this.texture = Lib.check.totalStringCheck(opts.texture, "");
        var group = Lib.check.objHasMethod(opts.group, "create") ? opts.group : undefined;
        var anchor = Lib.check.objHasProps(opts.anchor, ["x", "y"]) ? opts.anchor : new Phaser.Point(0, 0);
        var blendMode = Lib.check.isNumber(opts.blendMode) ? opts.blendMode : Phaser.blendModes.NORMAL;


        // this.obj = game.add.image(this.x, this.y, this.texture);
        this.obj = Lib.UTILS.createImageInGroup(this.x, this.y, this.texture, group);
        //set anchor to 0.5 for proper behaviour 
        this.obj.anchor.setTo(anchor.x, anchor.y);
        this.obj.blendMode = blendMode;

        // STANDARD REPO CODE - ----------------------------------------
        repo.addObj(this);
        //REQUIRED-----------------
        this.destroy = function() {
                repo.removeObj(this);
                self.obj.destroy();
            }
            //OPTIONAL--------------------
        this._update = function() {
            // p("Im a button");
        }
    }

    obj.Image = Image;

    //BUTTON--------------------------------------------------------------
    function Button(opts) {
        var self = this;
        var opts = opts || {}; //default
        this.x = Lib.check.isNumber(opts.x) ? opts.x : 0;
        this.y = Lib.check.isNumber(opts.y) ? opts.y : 0;
        this.texture = Lib.check.totalStringCheck(opts.texture, Lib.DEFS.TEX.STD_PLAY_BUTTON);
        this.callback = !Lib.check.isFunction(opts.callback) ? (function() {
            Lib.glob.warn("EmptyFunction");
        }) : opts.callback; //empty function is no callback provided
        this.context = !Lib.check.isObject(opts.context) ? {} : opts.context;
        var group = Lib.check.objHasMethod(opts.group, "create") ? opts.group : undefined;
        this.enabled = Lib.check.isBoolean(opts.enabled) ? opts.enabled : true;
        var anchor = Lib.check.objHasProps(opts.anchor, ["x", "y"]) ? opts.anchor : new Phaser.Point(0.5,0.5);


        //warning if context and callback weren't provided
        if (!Lib.check.theseArgsExist(opts.callback, opts.context) || !Lib.check.isFunction(opts.callback) || !Lib.check.isObject(opts.context)) {
            Lib.glob.warn("The callback or the context for this button are missing. Using empty.");
        }


        //this gets called when button is clicked. He in turn calls onClickFunc which simply calls the main callback.
        var localCallback = function() {
            p("Local Callback Called");
            if(!self.enabled){
                return 0;
            }
            this.onClickFunc(this.obj, this);
        }

        this.mainCallback = function() {
            self.callback.call(this.context);
            // this.obj.destroy();
            // this.destroy();
        }

        //replace it with a custom function which will do something and than call main callback.
        this.onClickFunc = function(context) {
            p("No fx. Executing main callback");
            // context.callback.call(context.context);
            self.mainCallback();
        }

        //create button
        // this.obj = game.add.button(this.x, this.y, this.texture, localCallback, this);
        this.obj = Lib.UTILS.createButtonInGroup(this.x, this.y, this.texture,localCallback,this,group);
        //set anchor to 0.5 for proper behaviour 
        // this.obj.anchor.setTo(0.5, 0.5);
        this.obj.anchor.setTo(anchor.x, anchor.y);

        var self = this;
        this.obj.setInput = function(val) {
            self.obj.input.enabled = val;
        }

        // STANDARD REPO CODE - ----------------------------------------
        repo.addObj(this);
        //REQUIRED-----------------
        this.destroy = function() {
                repo.removeObj(this);
                self.obj.destroy();
            }
            //OPTIONAL--------------------
        this._update = function() {
            // p("Im a button");
        }

    }

    obj.Button = Button;

    //BUTTON TOGGLE --------------------------------------------------------------
    //first argument for the callback is the toggle val. Starting wit true as default
    function ButtonToggle(opts) {
        var self = this;
        var opts = opts || {}; //default
        this.x = Lib.check.isNumber(opts.x) ? opts.x : 0;
        this.y = Lib.check.isNumber(opts.y) ? opts.y : 0;
        this.textureTrue = Lib.check.totalStringCheck(opts.textureTrue, Lib.DEFS.TEX.STD_TOGGLE_BUTTON_A);
        this.textureFalse = Lib.check.totalStringCheck(opts.textureFalse, Lib.DEFS.TEX.STD_TOGGLE_BUTTON_B);
        this.callback = !Lib.check.isFunction(opts.callback) ? (function() {
            Lib.glob.warn("EmptyFunction");
        }) : opts.callback; //empty function is no callback provided
        this.context = !Lib.check.isObject(opts.context) ? {} : opts.context;
        var group = Lib.check.objHasMethod(opts.group, "create") ? opts.group : undefined;
        //---------------
        this.toggleVal = true;
        this.enabled = Lib.check.isBoolean(opts.enabled) ? opts.enabled : true;


        //warning if context and callback weren't provided
        if (!Lib.check.theseArgsExist(opts.callback, opts.context) || !Lib.check.isFunction(opts.callback) || !Lib.check.isObject(opts.context)) {
            Lib.glob.warn("The callback or the context for this button are missing. Using empty.");
        }


        //this gets called when button is clicked. He in turn calls onClickFunc which simply calls the main callback.
        var localCallback = function() {
            p("Local Callback Called");
            if(!self.enabled){
                return 0;
            }
            this.onClickFunc(this.obj, this);
        }

        this.mainCallback = function() {
            //change toggle val
            self.toggleVal = !this.toggleVal;
            //change texture
            if (self.toggleVal) {
                self.obj.loadTexture(self.textureTrue);
            }
            else {
                self.obj.loadTexture(self.textureFalse);
            }

            self.callback.call(self.context, self.toggleVal);
            // this.obj.destroy();
            // this.destroy();
        }

        //replace it with a custom function which will do something and than call main callback.
        this.onClickFunc = function(context) {
            p("No fx. Executing main callback");
            // context.callback.call(context.context);
            self.mainCallback();
        }

        //create button
        // this.obj = game.add.button(this.x, this.y, this.textureTrue, localCallback, this);
        this.obj = Lib.UTILS.createButtonInGroup(this.x, this.y, this.textureTrue,localCallback,this,group);
        p(this.obj);
        //set anchor to 0.5 for proper behaviour 
        this.obj.anchor.setTo(0.5, 0.5);

        var self = this;
        this.obj.setInput = function(val) {
            self.obj.input.enabled = val;
        }

        // STANDARD REPO CODE - ----------------------------------------
        repo.addObj(this);
        //REQUIRED-----------------
        this.destroy = function() {
                repo.removeObj(this);
                self.obj.destroy();
            }
            //OPTIONAL--------------------
        this._update = function() {
            // p("Im a button");
        }

    }

    obj.ButtonToggle = ButtonToggle;

    return obj;
}());
